///////////////////////////////////////////////////////////////////////
//                                                                   //
//   MainForm.cs                                                     //
//                                                                   //
//   C# source code for main application window.                     //
//                                                                   //
//   Copyright (C) 2007, by Matt Roberts, KK5JY,                     //
//   All rights reserved.                                            //
//                                                                   //
///////////////////////////////////////////////////////////////////////

// system modules
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace SimpleImageViewer {
	//
	//
	//   class MainForm - form upon which the rest of the application is built
	//
	//
	public partial class MainForm : Form {
		#region Internal Fields
		private string[] loopImages;
		private Timer loopTimer;
		private int loopIndex;
		private int loopInterval;
		#endregion

		#region Public Instance Constructors

		public MainForm() {
			// initialize designer half of form
			InitializeComponent();

			// initialize loop stuff
			loopImages = null;
			loopInterval = 500;

			// tie key handler(s)
			imagePanel.KeyDown += this.KeyDownHandler;

			// aspect handling
			imagePanel.PreserveAspectRatio = m_PreserveAspect.Checked;
		}

		#endregion

		#region Public Properties

		public bool IsFileOpen {
			get { return imagePanel.Image != null; }
		}

		public bool Looping {
			get { return loopTimer != null && loopImages.Length > 0; }
		}

		#endregion

		#region Public Methods

		public void SetImage(string file) {
			Cursor c = Cursor.Current;
			Cursor.Current = Cursors.WaitCursor;
			try {
				Image oldImage = imagePanel.Image;
				imagePanel.Image = new Bitmap(file);
				if (oldImage != null)
					oldImage.Dispose();
			} finally {
				Cursor.Current = c;
			}
		}

		public void SetImage(Image image) {
			Cursor c = Cursor.Current;
			Cursor.Current = Cursors.WaitCursor;
			try {
				Image oldImage = imagePanel.Image;
				imagePanel.Image = image;
				if (oldImage != null)
					oldImage.Dispose();
			} finally {
				Cursor.Current = c;
			}
		}

		public void AdjustMenus() {
			m_FileOpen.Enabled = Looping;
		}

		#endregion

		#region Event Handlers

		private void KeyDownHandler(object sender, KeyEventArgs e) {
			// catch loop control keys
			if (Looping) {
				switch (e.KeyCode) {
					case Keys.Space:
						loopTimer.Enabled = !loopTimer.Enabled;
						e.Handled = true;
						break;
					case Keys.Escape:
						loopTimer.Stop();
						loopTimer.Dispose();
						loopTimer = null;
						e.Handled = true;
						progressBar.Value = 0;
						statusLabel.Text = "";
						break;
					case Keys.Left:
						if (!loopTimer.Enabled) {
							--loopIndex;
							if (loopIndex < 0)
								loopIndex = loopImages.Length - 1;
							SetImage(loopImages[loopIndex]);
						}
						break;
					case Keys.Right:
						if (!loopTimer.Enabled) {
							++loopIndex;
							if (loopIndex >= loopImages.Length)
								loopIndex = 0;
							SetImage(loopImages[loopIndex]);
						}
						break;
				}
			}
		}

		private void FileMenuOpen(object sender, EventArgs e) {
			OpenFileDialog dialog = new OpenFileDialog();
			dialog.Multiselect = false;
			try {
				DialogResult result = dialog.ShowDialog();
				if (result == DialogResult.OK) {
					SetImage(dialog.FileName);
				}
			} finally {
				dialog.Dispose();
			}
		}

		private void FileMenuClose(object sender, EventArgs e) {
			if (IsFileOpen) {
				imagePanel.Image = null;
			} else {
				FileMenuExit(sender, e);
			}
		}

		private void FileMenuExit(object sender, EventArgs e) {
			Close();
		}

		private void OptionsMenuPreserveAspect(object sender, EventArgs e) {
			m_PreserveAspect.Checked =
				!m_PreserveAspect.Checked;
			imagePanel.PreserveAspectRatio =
				m_PreserveAspect.Checked;
		}

		private void FileMenuLoopFiles(object sender, EventArgs e) {
			OpenFileDialog dialog = new OpenFileDialog();
			dialog.Multiselect = true;
			dialog.CheckFileExists = false;
			dialog.CheckPathExists = false;
			try {
				DialogResult result = dialog.ShowDialog();
				if (result != DialogResult.OK)
					return;

				if (loopTimer != null) {
					loopTimer.Enabled = false;
					loopTimer.Dispose();
					loopTimer = null;
					progressBar.Value = 0;
					statusLabel.Text = "";
				}

				loopImages = dialog.FileNames;
				loopIndex = -1;
				loopTimer = new Timer();
				loopTimer.Interval = loopInterval;
				loopTimer.Tick += this.OnTick;
				loopTimer.Enabled = true;
				statusLabel.Text = String.Format("{0} images selected.", loopImages.Length);
				progressBar.Value = 0;
			} finally {
				dialog.Dispose();
			}
		}

		private void OnTick(object sender, EventArgs e) {
			// disable the timer
			loopTimer.Enabled = false;

			if (++loopIndex >= loopImages.Length) 
				loopIndex = 0;

			Bitmap newImage = null;
			bool doContinue = true;
			do {
				try {
					newImage = new Bitmap(loopImages[loopIndex]);
					doContinue = false;
					SetImage(newImage);
				} catch (Exception) {
					doContinue = true;
					if (newImage != null) {
						newImage.Dispose();
					}
				}
			} while (doContinue);

			// update the progress bar
			progressBar.Value = (100 * loopIndex / loopImages.Length);

			// reenable the timer
			loopTimer.Enabled = true;
		}

		private void OptionsMenuSetLoopInterval(object sender, EventArgs e) {
			LoopIntervalDialog dialog = new LoopIntervalDialog();
			dialog.Interval = loopInterval;
			try {
				DialogResult result = dialog.ShowDialog();
				if (result != DialogResult.OK)
					return;

				if (dialog.Interval != 0) {
					loopInterval = dialog.Interval;
					if (loopTimer != null)
						loopTimer.Interval = loopInterval;
				}

			} finally {
				dialog.Dispose();
			}
		}

		protected override void OnClosing(CancelEventArgs e) {
			if (loopTimer != null) {
				loopTimer.Stop();
			}

			// chain to base
			base.OnClosing(e);
		}

		#endregion Event Handlers
	}
}